#ifndef OAS_AUDIO_H

#define OAS_AUDIO_H

#include <windows.h>

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

// commands
#define OAS_MSG_CMD_INPUT_OPEN              WM_APP
#define OAS_MSG_CMD_STOP                    WM_APP+1
#define OAS_MSG_CMD_PAUSE                   WM_APP+2
#define OAS_MSG_CMD_PLAY                    WM_APP+3
#define OAS_MSG_CMD_SEEK                    WM_APP+4
#define OAS_MSG_CMD_OUTPUT_OPEN             WM_APP+5
#define OAS_MSG_CMD_SET_OUTPUT_VOLUME       WM_APP+6
#define OAS_MSG_CMD_ADD_OUTPUT_FILTER       WM_APP+7
#define OAS_MSG_CMD_PRELOAD_BUFFERS         WM_APP+8
#define OAS_MSG_CMD_SET_INPUT_POSN_RANGE    WM_APP+9
#define OAS_MSG_CMD_OUTPUT_RESET            WM_APP+10
#define OAS_MSG_CMD_OUTPUT_DRAIN            WM_APP+11
#define OAS_MSG_CMD_PING                    WM_APP+12
// from 0.40 onwards
#define OAS_MSG_CMD_INPUT_CLOSE             WM_APP+13
#define OAS_MSG_CMD_SET_DURATION            WM_APP+14
#define OAS_MSG_CMD_SET_INPUT_TIME_INTERVAL WM_APP+15
// from 0.73 onwards
#define OAS_MSG_CMD_SET_OUTPUT_FREQUENCY    WM_APP+16
// from 0.9 onwards - internal use only
#define OAS_MSG_INTERNAL_ERROR              WM_APP+17

// status messages back
#define OAS_MSG_NTFY_NACK             WM_APP
#define OAS_MSG_NTFY_ACK              WM_APP+1
#define OAS_MSG_NTFY_STATE            WM_APP+2
#define OAS_MSG_NTFY_INPUT_POSN       WM_APP+3
#define OAS_MSG_NTFY_INPUT_TIME       WM_APP+4
#define OAS_MSG_NTFY_STREAM_DURATION  WM_APP+5
#define OAS_MSG_NTFY_PONG             WM_APP+6
#define OAS_MSG_NTFY_PRELOAD_STATE    WM_APP+7
#define OAS_MSG_NTFY_ERROR            WM_APP+8

// error codes returned as lParam of a NACK response
#define OAS_ERROR_INPUT_OPEN 1
#define OAS_ERROR_OUTPUT_OPEN 2
#define OAS_ERROR_PRELOAD_NOT_DSOUND 3
#define OAS_ERROR_ADD_OUTPUT_FILTER 4
#define OAS_ERROR_CODEC_FAILURE 5  // from 0.30: this error is no longer generated
#define OAS_ERROR_INVALID_PARMS 6
// from 0.40 onwards
#define OAS_ERROR_NOT_OPEN 7

// error codes returned as lParam of an ERROR response
// these are serious (and probably fatal) conditions
// from 0.72 onwards
#define OAS_ERROR_EXCEPTION 8
// from 0.9 onwards
#define OAS_ERROR_DSOUND_UNSUPP_FORMAT 9
// from 0.94 onwards
#define OAS_ERROR_DSOUND_BUFFER_LOCK 10
#define OAS_ERROR_ASYNCIO 11

// overall count
#define OAS_ERROR_COUNT 12

// this is the buffer latency (in ms) of the direct sound
// buffers
#define OAS_DSOUND_LATENCY 1000

// lparam codes on exception raise
#define OAS_EXCEPTION_CORE 0
#define OAS_EXCEPTION_AIO 1
#define OAS_EXCEPTION_DSOUND 2

typedef enum {
    OAS_PLAYER_STATE_STOPPED,
    OAS_PLAYER_STATE_PLAYING,
    OAS_PLAYER_STATE_PAUSED,
    OAS_PLAYER_STATE_EOS
} OAS_PlayerState;

typedef enum {
    OAS_PRELOAD_STATE_LOADED,
    OAS_PRELOAD_STATE_NOTLOADED,
    OAS_PRELOAD_STATE_NO_INPUT_STREAM } OASPreloadState ;

typedef struct {
  BOOL (*OutputCreate)( void **OutputInst, void *Params ) ;
  void (*OutputDelete)( void *OutputInst ) ;
  BOOL (*OutputOpen) ( void *OutputInst ) ;
  void (*OutputClose) ( void *OutputInst ) ;
  void (*OutputWrite) ( void *OutputInst,
                        void *SampleBfr,
                        unsigned long Size,
                        unsigned short BytesPerSample,
                        unsigned short Channels,
                        unsigned long SampleRate ) ;
  // new in 0.93
  BOOL (*OutputCmd) ( void *OutputInst, unsigned long Cmd, unsigned long Param ) ;
} OutputModule ;

typedef struct OutputFilterNode OutputFilterNode ;

typedef struct {
  BOOL (*FilterCreate)( void **FilterInst, void *Params) ;
  void (*FilterDelete)(void *FilterInst) ;
  BOOL (*FilterOpen)(void *FilterInst) ;
  void (*FilterClose)(void *FilterInst) ;
  void (*FilterWrite)(void *FilterInst,
                      void *SampleBfr,
                      unsigned long Size,
                      unsigned short BytesPerSample,
                      unsigned short Channels,
                      unsigned long SampleRate ) ;
} OutputFilterModule ;

struct OutputFilterNode {
    OutputFilterModule VectorTable ;
    void               *FilterInst ;
    OutputFilterNode   *Next;
    OutputFilterNode   *Prev;
};

typedef struct {
  unsigned long  SampleRate ;
  unsigned long  TrackLength ;
  unsigned short BitRate ;
  unsigned short Channels ;
  BOOL           Supported ; } MPEGTrackInfoType ;

void *OASAudioPlayerCreate ( HWND hWnd ) ;

void OASAudioPlayerDestroy ( void *Player ) ;

char **OASAudioEnumerateDevices ( unsigned *Count ) ;

void OASAudioFreeDeviceEnums ( char **DevEnums ) ;

void OASAudioMessageSend ( void *Player,
                           unsigned long Msg,
                           ... ) ;

unsigned long OASAudioGetMP3TrackLength ( char *FileName ) ;

BOOL OASAudioGetMP3TrackInfo ( char *FileName, MPEGTrackInfoType *pTrackInfo ) ;

// new.. in 0.84
void OASAudioFormatErrorString ( int Cmd,
                                 int ErrorCode,
                                 char *Buffer ) ;

#ifdef __cplusplus
}
#endif /*__cplusplus*/

#endif

